#include <QtCore/QCoreApplication>
#include <QtCore/QThread>

#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusAbstractInterface>
#include <QtDBus/QDBusPendingReply>

class TestProxy: public QDBusAbstractInterface
{
public:
  TestProxy(const QString &service, const QString &path, const QDBusConnection &connection )
      : QDBusAbstractInterface(service, path, "org.freedesktop.DBus", connection, NULL)
  { }
};

void MyThreadFunction() {
  printf("Thread starting\n");
  QDBusAbstractInterface *pi = new TestProxy("org.freedesktop.DBus", "/", QDBusConnection::sessionBus() );
  int iterations = 0;
  QLatin1String method("ListNames");
  QList<QVariant> argumentList;
  while( 1 ) {
    printf("Iter...%d\n", ++iterations );
    QDBusPendingReply<> reply = pi->asyncCallWithArgumentList(method, argumentList);
	  reply.waitForFinished();
    if( !reply.isValid() ) {
      printf( "Failed\n");
    }
  }
}


class MyThread : public QThread
 {
 public:
  void run()
  {
    MyThreadFunction();
  }
};


//  Set to 0 to test in the main QT thread...higher==crash faster!
static const int THREAD_COUNT = 10;

int main(int argc, char* argv[])
{
  QCoreApplication app(argc, argv);
  if (! QDBusConnection::sessionBus().isConnected()) {
      printf("Unable to connect to DBus. The test requires a running dbus daemon.");
      return 1;
  }
  if( THREAD_COUNT  ) {
    for( int n =0; n < THREAD_COUNT; n++ ) {
      MyThread *p = new MyThread(); p->start();
    }
  } else {
    MyThreadFunction();
  }

  return app.exec();
}
